var FullCalendarVue = (function (exports, core, Vue) {
    'use strict';

    function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

    var Vue__default = /*#__PURE__*/_interopDefaultLegacy(Vue);

    const OPTION_IS_COMPLEX = {
        headerToolbar: true,
        footerToolbar: true,
        events: true,
        eventSources: true,
        resources: true
    };

    // TODO: add types!
    /*
    works with objects and arrays
    */
    function shallowCopy(val) {
        if (typeof val === 'object') {
            if (Array.isArray(val)) {
                val = Array.prototype.slice.call(val);
            }
            else if (val) { // non-null
                val = { ...val };
            }
        }
        return val;
    }
    function mapHash(input, func) {
        const output = {};
        for (const key in input) {
            if (input.hasOwnProperty(key)) {
                output[key] = func(input[key], key);
            }
        }
        return output;
    }

    /*
    wrap it in an object with a `vue` key, which the custom content-type handler system will look for
    */
    function wrapVDomGenerator(vDomGenerator) {
        return function (props) {
            return { vue: vDomGenerator(props) };
        };
    }
    function createVueContentTypePlugin(parent) {
        return core.createPlugin({
            contentTypeHandlers: {
                vue: () => buildVDomHandler(parent), // looks for the `vue` key
            }
        });
    }
    function buildVDomHandler(parent) {
        let currentEl;
        let v; // the Vue instance
        function render(el, vDomContent) {
            if (currentEl !== el) {
                if (currentEl && v) { // if changing elements, recreate the vue
                    v.$destroy();
                }
                currentEl = el;
            }
            if (!v) {
                v = initVue(vDomContent, parent);
                // vue's mount method *replaces* the given element. create an artificial inner el
                let innerEl = document.createElement('span');
                el.appendChild(innerEl);
                v.$mount(innerEl);
            }
            else {
                v.content = vDomContent;
            }
        }
        function destroy() {
            if (v) { // needed?
                v.$destroy();
            }
        }
        return { render, destroy };
    }
    function initVue(initialContent, parent) {
        return new Vue__default["default"]({
            parent,
            data: {
                content: initialContent,
            },
            render(h) {
                let { content } = this;
                // the slot result can be an array, but the returned value of a vue component's
                // render method must be a single node.
                if (content.length === 1) {
                    return content[0];
                }
                else {
                    return h('span', {}, content);
                }
            }
        });
    }

    var FullCalendarComponent = Vue.defineComponent({
        name: 'FullCalendar',
        props: {
            options: {
                type: Object,
                default: () => ({})
            }
        },
        render(h) {
            return h('div', {
                attrs: {
                    'data-fc-render-id': this.renderId
                }
            });
        },
        setup(props) {
            const renderId = Vue.ref(0);
            const instance = Vue.getCurrentInstance();
            const root = instance && instance.proxy;
            if (!root) {
                return {};
            }
            function buildOptions(suppliedOptions) {
                let internal = root && root.$options;
                suppliedOptions = suppliedOptions || {};
                return {
                    ...internal.scopedSlotOptions,
                    ...suppliedOptions,
                    plugins: (suppliedOptions.plugins || []).concat([
                        createVueContentTypePlugin(root)
                    ])
                };
            }
            function getApi() {
                return (root && root.$options).calendar;
            }
            Vue.onMounted(() => {
                let internal = root.$options;
                internal.scopedSlotOptions = mapHash(root.$scopedSlots, wrapVDomGenerator); // needed for buildOptions
                const calendar = new core.Calendar(root.$el, buildOptions(props.options));
                internal.calendar = calendar;
                calendar.render();
            });
            Vue.onBeforeUpdate(() => getApi().resumeRendering());
            Vue.onBeforeUnmount(() => getApi().destroy());
            Vue.watch(() => props.options, val => {
                let calendar = getApi();
                calendar.pauseRendering();
                calendar.resetOptions(buildOptions(val));
                renderId.value++; // will queue a rerender
            }, { deep: true });
            for (const complexOptionName in OPTION_IS_COMPLEX) {
                Vue.watch(() => props.options[complexOptionName], val => {
                    // unfortunately the handler is called with undefined if new props were set, but the complex one wasn't ever set
                    if (val !== undefined) {
                        let calendar = getApi();
                        calendar.pauseRendering();
                        calendar.resetOptions({
                            [complexOptionName]: shallowCopy(val)
                        }, true);
                        renderId.value++;
                    }
                });
            }
            return {
                getApi,
                renderId
            };
        }
    });

    exports["default"] = FullCalendarComponent;
    Object.keys(core).forEach(function (k) {
        if (k !== 'default' && !exports.hasOwnProperty(k)) Object.defineProperty(exports, k, {
            enumerable: true,
            get: function () { return core[k]; }
        });
    });

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({}, FullCalendar, Vue);
